# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt

import gocept.directory.interfaces
import grok
import netaddr
import os.path
import yaml


grok.context(gocept.directory.interfaces.IDirectory)


class AccessRing0(grok.Permission):
    grok.name('gocept.directory.AccessRing0')


class Ring0(grok.XMLRPC):

    # XXX Requires changes to our clients.
    # grok.require(AccessRing0)

    def __init__(self, context, request):
        self.dbdir = context.paltry
        self.node_db = os.path.join(self.dbdir, 'nodes')
        self.passwd_db = os.path.join(self.dbdir, 'passwords')
        self.people_db = os.path.join(self.dbdir, 'people', 'people.yml')
        self.groups_db = os.path.join(self.dbdir, 'people', 'groups.yml')
        self.location_db = os.path.join(self.dbdir, 'locations')

    @property
    def __nodes(self):
        for name in sorted(os.listdir(self.node_db)):
            if name.startswith('.'):
                continue
            node = yaml.load(open(os.path.join(self.node_db, name)))
            node['name'] = name
            # DEPRECATED: provide node['fqdn'] for backward compatibility
            # use node['name'] instead
            node['fqdn'] = name
            yield node

    def list_virtual_machines(self):
        result = []
        for node in self.__nodes:
            if node['parameters']['machine'] != 'virtual':
                continue
            result.append(node)
        return result

    def list_nodes(self):
        result = []
        for node in self.__nodes:
            result.append(node)
        return result

    def lookup_node(self, lookup_name):
        for node in self.__nodes:
            if node['name'] != lookup_name:
                continue
            return node
        raise KeyError(lookup_name)

    def get_password(self, service):
        if not service in os.listdir(self.passwd_db):
            raise KeyError(service)
        return open(os.path.join(self.passwd_db, service)).read().strip()

    def list_people(self):
        return yaml.load(open(self.people_db))

    def list_groups(self):
        return yaml.load(open(self.groups_db))

    def list_locations(self):
        locations = [l for l in os.listdir(self.location_db)
                     if not l.startswith('.')]
        return sorted(locations)

    def lookup_location(self, name):
        # DEPRECATED, use lookup_network instead
        return yaml.load(open(os.path.join(self.location_db, name)))

    def lookup_networks(self, location, af=''):
        networks = yaml.load(
            open(os.path.join(self.location_db, location)))['networks']
        # replace inner dicts with lists
        for vlan, nets in networks.items():
            if af:
                filtered_nets = [val for key, val in nets.items() if key == af]
            else:
                filtered_nets = nets.values()
            if filtered_nets:
                networks[vlan] = sorted(filtered_nets)
            else:
                del networks[vlan]
        return networks

    def list_networks(self):
        networks = []
        for location in os.listdir(self.location_db):
            if location.startswith('.'):
                continue
            for network in self.lookup_networks(location).values():
                networks.extend(network)
        return sorted(networks)

    def _build_address(self, nodeparams, network, vlan):
        if network.addr_type == 6:
            return network[nodeparams['id']]
        elif network.addr_type == 4:
            try:
                return network[nodeparams['ip4_%s' % vlan]]
            except KeyError:
                pass

    def lookup_addresses(self, nodename, vlan_filter='', af_filter=''):
        addresses = []
        node = self.lookup_node(nodename)
        params = node['parameters']
        for vlan, networks in self.lookup_networks(
                params['location'], af_filter).items():
            if vlan_filter and vlan != vlan_filter:
                continue
            for n in networks:
                network = netaddr.CIDR(n)
                addr = self._build_address(params, network, vlan)
                if addr:
                    addresses.append('%s/%i' % (str(addr), network.prefixlen))
        return sorted(addresses)

