# vim:fileencoding=utf-8
# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt

import gocept.directory.testing
import os.path
import unittest
import xmlrpclib
import zope.app.testing.functional
import zope.app.testing.xmlrpc


class TestRing0(zope.app.testing.functional.FunctionalTestCase):

    layer = gocept.directory.testing.DataLayer

    def setUp(self):
        self.directory = self.getRootFolder()['directory']
        self.directory.paltry = os.path.join(
            os.path.dirname(gocept.directory.__file__), 'testdb')
        self.rpc = zope.app.testing.xmlrpc.ServerProxy(
            'http://localhost/directory/')

    def test_password(self):
        self.assertEquals('nenpotgaf', self.rpc.get_password('ldap'))
        self.assertRaises(xmlrpclib.Fault, self.rpc.get_password, 'foobar')

    def test_people(self):
        people = self.rpc.list_people()
        self.assertEquals(40, len(people))
        self.assertEquals({
            'mail': 'ct@gocept.com',
            'pager': '+49 179 7808366',
            'uid': 'ctheune',
            'cn': 'Christian Theune'}, people['ctheune'])

    def test_groups(self):
        groups = self.rpc.list_groups()
        self.assertEquals(17, len(groups))
        self.assertEquals(['zagy', 'ctheune', 'nilo', 'ckauhaus'],
                          groups['admins'])

    def test_lookup_node(self):
        # node['fqdn'] is DEPRECATED
        self.assertEquals({
            'classes': ['role::puppetmaster', 'role::nagios',
                        'role::portage_binhost'],
            'name': 'stampy',
            'fqdn': 'stampy',
            'parameters': {
                'environment': 'testing',
                'contacts': 'ckauhaus, ctheune',
                'machine': 'physical',
                'location': 'newyork',
                'mac_ipmi': '00:1e:c9:ad:4a:a4',
                'mac_fe': '00:15:17:91:d2:f0',
                'mac_srv': '00:1e:c9:ad:4a:a0',
                'mac_sto': '00:1e:c9:ad:4a:a2',
                'fqdn_fe': 'monitor.whq.gocept.com',
                'ip4_ipmi': 9,
                'ip4_fe': 3,
                'ip4_srv': 10,
                'ip4_sto': 10,
                'id': 4099}},
            self.rpc.lookup_node('stampy'))

    def test_lookup_invalid_node_should_fail(self):
        self.assertRaises(xmlrpclib.Fault, self.rpc.lookup_node, 'foobar')

    def test_list_virtual_machines(self):
        vms = self.rpc.list_virtual_machines()
        self.assertEquals(1, len(vms))
        self.assertEquals('test00', vms[0]['name'])

    def test_lookup_location(self):
        # lookup_location is DEPRECATED
        self.assertEquals(
            {'networks': {'sto': {'ipv6': '2001:470:9aaf:44::/64'},
                          'srv': {'ipv4': '172.20.3.0/24',
                                  'ipv6': '2001:470:9aaf:43::/64'},
                          'fe': {'ipv4': '172.20.2.0/24', 'ipv6':
                                 '2001:470:9aaf:42::/64'},
                          'mgm': {'ipv4': '172.20.1.0/24'}}},
            self.rpc.lookup_location('jamaica'))

    def test_list_networks(self):
        self.assertEquals(
            ['172.20.1.0/24', '172.20.2.0/24', '172.20.3.0/24',
             '2001:470:9aaf:42::/64',
             '2001:470:9aaf:43::/64', '2001:470:9aaf:44::/64',
             '2002:470:9aaf:41::/64', '2002:470:9aaf:42::/64',
             '2002:470:9aaf:43::/64', '2002:470:9aaf:44::/64',
             '231.20.1.0/24', '231.20.2.0/24', '231.20.3.0/24', '231.20.4.0/24',
             ],
            self.rpc.list_networks())

    def test_list_nodes(self):
        nodes = self.rpc.list_nodes()
        self.assertEquals(['otto', 'stampy', 'test00'],
                          [n['name'] for n in nodes])

    def test_list_locations(self):
        self.assertEquals(
            ['jamaica', 'newyork'],
            self.rpc.list_locations())

    def test_lookup_networks(self):
        self.assertEquals(
            {'sto': ['2001:470:9aaf:44::/64'],
             'srv': ['172.20.3.0/24', '2001:470:9aaf:43::/64'],
             'fe': ['172.20.2.0/24', '2001:470:9aaf:42::/64'],
             'mgm': ['172.20.1.0/24']},
            self.rpc.lookup_networks('jamaica'))

    def test_lookup_networks_with_af(self):
        self.assertEquals(
            {'srv': ['172.20.3.0/24'],
             'fe': ['172.20.2.0/24'],
             'mgm': ['172.20.1.0/24']},
            self.rpc.lookup_networks('jamaica', 'ipv4'))

    def test_lookup_addresses(self):
        self.assertEquals(
            ['2002:470:9aaf:41::1003/64', '2002:470:9aaf:42::1003/64',
             '2002:470:9aaf:43::1003/64', '2002:470:9aaf:44::1003/64',
             '231.20.2.3/24', '231.20.3.10/24', '231.20.4.10/24'],
            self.rpc.lookup_addresses('stampy'))

    def test_lookup_addresses_with_vlan(self):
        self.assertEquals(
            ['2002:470:9aaf:42::1003/64', '231.20.2.3/24'],
            self.rpc.lookup_addresses('stampy', 'fe'))

    def test_lookup_addresses_with_af(self):
        self.assertEquals(
            ['2002:470:9aaf:41::1003/64', '2002:470:9aaf:42::1003/64',
             '2002:470:9aaf:43::1003/64', '2002:470:9aaf:44::1003/64'],
            self.rpc.lookup_addresses('stampy', '', 'ipv6'))


def test_suite():
    return unittest.makeSuite(TestRing0)
