# Copyright (c) 2008-2009 gocept gmbh & co. kg
# See also LICENSE.txt

import grok
import zope.component

import gocept.directory.interfaces
import gocept.directory.resourcegroup


class ExportPOSIX(grok.Model):
    """Export directory data as POSIX passwd/shadow/group files."""

    def iter_unix_accounts(self):
        unix_accounts = zope.component.getUtility(
            gocept.directory.interfaces.IUnixAccounts)
        return unix_accounts.values()

    def iter_groups(self):
        customers = zope.component.getUtility(
            gocept.directory.interfaces.ICustomers)
        yield gocept.directory.resourcegroup.defaultgroup
        for customer in customers.values():
            container = zope.component.getUtility(
                gocept.directory.interfaces.IResourceGroups, context=customer)
            for resourcegroup in container.values():
                yield resourcegroup

    def iter_usernames(self, group):
        for person in group.persons:
            try:
                account = gocept.directory.interfaces.IUnixAccount(person)
            except TypeError:
                continue
            yield account.username


class Passwd(grok.View):

    def render(self):
        lines = []
        for account in self.context.iter_unix_accounts():
            lines.append(
                '%(username)s:*:%(uid)s:%(gid)s:%(gecos)s:'
                '/home/%(username)s:/bin/bash\n' % dict(
                username=account.username,
                uid=account.uid,
                gid=gocept.directory.resourcegroup.defaultgroup.gid,
                gecos=account.gecos,
                ))
        return ''.join(lines)


class Shadow(grok.View):

    def render(self):
        lines = []
        for account in self.context.iter_unix_accounts():
            lines.append('%(username)s:%(password)s:0:0:99999:7:::\n' % dict(
                username=account.username,
                password=account.crypted_password,
            ))
        return ''.join(lines)


class Group(grok.View):

    def render(self):
        lines = []
        for group in self.context.iter_groups():
            lines.append(
                '%(group)s:x:%(gid)s:%(members)s\n' % dict(
                    group=group.group,
                    gid=group.gid,
                    members=','.join(self.context.iter_usernames(group))
                ))
        return ''.join(lines)
